from hurodes.hrdf.infos.actuator import ActuatorInfo
from hurodes.hrdf.infos.body import BodyInfo
from hurodes.hrdf.infos.simple_geom import SimpleGeomInfo
from hurodes.hrdf.infos.joint import JointInfo
from hurodes.hrdf.infos.mesh import MeshInfo

INFO_CLASS_DICT = {
    "actuator": ActuatorInfo,
    "body": BodyInfo,
    "simple_geom": SimpleGeomInfo,
    "joint": JointInfo,
    "mesh": MeshInfo
}
